"""Python bindings to the Zstandard (zstd) compression library (RFC-8878)."""

import enum
from _typeshed import ReadableBuffer
from collections.abc import Iterable, Mapping
from compression.zstd._zstdfile import ZstdFile, open
from typing import Final, final

import _zstd
from _zstd import ZstdCompressor, ZstdDecompressor, ZstdDict, ZstdError, get_frame_size, zstd_version

__all__ = (
    # compression.zstd
    "COMPRESSION_LEVEL_DEFAULT",
    "compress",
    "CompressionParameter",
    "decompress",
    "DecompressionParameter",
    "finalize_dict",
    "get_frame_info",
    "Strategy",
    "train_dict",
    # compression.zstd._zstdfile
    "open",
    "ZstdFile",
    # _zstd
    "get_frame_size",
    "zstd_version",
    "zstd_version_info",
    "ZstdCompressor",
    "ZstdDecompressor",
    "ZstdDict",
    "ZstdError",
)

zstd_version_info: Final[tuple[int, int, int]]
COMPRESSION_LEVEL_DEFAULT: Final = _zstd.ZSTD_CLEVEL_DEFAULT

class FrameInfo:
    """Information about a Zstandard frame."""

    __slots__ = ("decompressed_size", "dictionary_id")
    decompressed_size: int
    dictionary_id: int
    def __init__(self, decompressed_size: int, dictionary_id: int) -> None: ...

def get_frame_info(frame_buffer: ReadableBuffer) -> FrameInfo:
    """Get Zstandard frame information from a frame header.

    *frame_buffer* is a bytes-like object. It should start from the
    beginning of a frame, and needs to include at least the frame header
    (6 to 18 bytes).

    The returned FrameInfo object has two attributes.
    'decompressed_size' is the size in bytes of the data in the frame when
    decompressed, or None when the decompressed size is unknown.
    'dictionary_id' is an int in the range (0, 2**32). The special value 0
    means that the dictionary ID was not recorded in the frame header,
    the frame may or may not need a dictionary to be decoded,
    and the ID of such a dictionary is not specified.
    """

def train_dict(samples: Iterable[ReadableBuffer], dict_size: int) -> ZstdDict:
    """Return a ZstdDict representing a trained Zstandard dictionary.

    *samples* is an iterable of samples, where a sample is a bytes-like
    object representing a file.

    *dict_size* is the dictionary's maximum size, in bytes.
    """

def finalize_dict(zstd_dict: ZstdDict, /, samples: Iterable[ReadableBuffer], dict_size: int, level: int) -> ZstdDict:
    """Return a ZstdDict representing a finalized Zstandard dictionary.

    Given a custom content as a basis for dictionary, and a set of samples,
    finalize *zstd_dict* by adding headers and statistics according to the
    Zstandard dictionary format.

    You may compose an effective dictionary content by hand, which is used
    as basis dictionary, and use some samples to finalize a dictionary.  The
    basis dictionary may be a "raw content" dictionary.  See *is_raw* in
    ZstdDict.

    *samples* is an iterable of samples, where a sample is a bytes-like
    object representing a file.
    *dict_size* is the dictionary's maximum size, in bytes.
    *level* is the expected compression level. The statistics for each
    compression level differ, so tuning the dictionary to the compression
    level can provide improvements.
    """

def compress(
    data: ReadableBuffer,
    level: int | None = None,
    options: Mapping[int, int] | None = None,
    zstd_dict: ZstdDict | tuple[ZstdDict, int] | None = None,
) -> bytes:
    """Return Zstandard compressed *data* as bytes.

    *level* is an int specifying the compression level to use, defaulting to
    COMPRESSION_LEVEL_DEFAULT ('3').
    *options* is a dict object that contains advanced compression
    parameters. See CompressionParameter for more on options.
    *zstd_dict* is a ZstdDict object, a pre-trained Zstandard dictionary.
    See the function train_dict for how to train a ZstdDict on sample data.

    For incremental compression, use a ZstdCompressor instead.
    """

def decompress(
    data: ReadableBuffer, zstd_dict: ZstdDict | tuple[ZstdDict, int] | None = None, options: Mapping[int, int] | None = None
) -> bytes:
    """Decompress one or more frames of Zstandard compressed *data*.

    *zstd_dict* is a ZstdDict object, a pre-trained Zstandard dictionary.
    See the function train_dict for how to train a ZstdDict on sample data.
    *options* is a dict object that contains advanced compression
    parameters. See DecompressionParameter for more on options.

    For incremental decompression, use a ZstdDecompressor instead.
    """

@final
class CompressionParameter(enum.IntEnum):
    """Compression parameters."""

    compression_level = _zstd.ZSTD_c_compressionLevel
    window_log = _zstd.ZSTD_c_windowLog
    hash_log = _zstd.ZSTD_c_hashLog
    chain_log = _zstd.ZSTD_c_chainLog
    search_log = _zstd.ZSTD_c_searchLog
    min_match = _zstd.ZSTD_c_minMatch
    target_length = _zstd.ZSTD_c_targetLength
    strategy = _zstd.ZSTD_c_strategy
    enable_long_distance_matching = _zstd.ZSTD_c_enableLongDistanceMatching
    ldm_hash_log = _zstd.ZSTD_c_ldmHashLog
    ldm_min_match = _zstd.ZSTD_c_ldmMinMatch
    ldm_bucket_size_log = _zstd.ZSTD_c_ldmBucketSizeLog
    ldm_hash_rate_log = _zstd.ZSTD_c_ldmHashRateLog
    content_size_flag = _zstd.ZSTD_c_contentSizeFlag
    checksum_flag = _zstd.ZSTD_c_checksumFlag
    dict_id_flag = _zstd.ZSTD_c_dictIDFlag
    nb_workers = _zstd.ZSTD_c_nbWorkers
    job_size = _zstd.ZSTD_c_jobSize
    overlap_log = _zstd.ZSTD_c_overlapLog
    def bounds(self) -> tuple[int, int]:
        """Return the (lower, upper) int bounds of a compression parameter.

        Both the lower and upper bounds are inclusive.
        """

@final
class DecompressionParameter(enum.IntEnum):
    """Decompression parameters."""

    window_log_max = _zstd.ZSTD_d_windowLogMax
    def bounds(self) -> tuple[int, int]:
        """Return the (lower, upper) int bounds of a decompression parameter.

        Both the lower and upper bounds are inclusive.
        """

@final
class Strategy(enum.IntEnum):
    """Compression strategies, listed from fastest to strongest.

    Note that new strategies might be added in the future.
    Only the order (from fast to strong) is guaranteed,
    the numeric value might change.
    """

    fast = _zstd.ZSTD_fast
    dfast = _zstd.ZSTD_dfast
    greedy = _zstd.ZSTD_greedy
    lazy = _zstd.ZSTD_lazy
    lazy2 = _zstd.ZSTD_lazy2
    btlazy2 = _zstd.ZSTD_btlazy2
    btopt = _zstd.ZSTD_btopt
    btultra = _zstd.ZSTD_btultra
    btultra2 = _zstd.ZSTD_btultra2



"""Functions to convert between Python values and C structs.
Python bytes objects are used to hold the data representing the C struct.
The format string (explained below) describes the layout of data
in the C struct.

The optional first format char indicates byte order, size and alignment:
  @: native order, size & alignment (default)
  =: native order, std. size & alignment
  <: little-endian, std. size & alignment
  >: big-endian, std. size & alignment
  !: same as >

The remaining characters indicate types of args and must match exactly;
these can be preceded by a decimal repeat count:
  x: pad byte (no data); c: char; b: signed byte; B: unsigned byte;
  ?: _Bool; h: short; H: unsigned short; i: int; I: unsigned int;
  l: long; L: unsigned long; q: long long; Q: unsigned long long;
  f: float; d: double; e: half-float;
  F: float complex; D: double complex.
Special cases (preceding decimal count indicates length):
  s: byte string (array of char); p: Pascal string (with count byte).
Special cases (only available in native format):
  n: ssize_t; N: size_t;
  P: an integer type that is wide enough to hold a pointer.
Whitespace between formats is ignored.

The variable struct.error is an exception raised on errors.
"""

from _struct import *

__all__ = ["calcsize", "pack", "pack_into", "unpack", "unpack_from", "iter_unpack", "Struct", "error"]

class error(Exception): ...



"""Ttk wrapper.

This module provides classes to allow using Tk themed widget set.

Ttk is based on a revised and enhanced version of
TIP #48 (http://tip.tcl.tk/48) specified style engine.

Its basic idea is to separate, to the extent possible, the code
implementing a widget's behavior from the code implementing its
appearance. Widget class bindings are primarily responsible for
maintaining the widget state and invoking callbacks, all aspects
of the widgets appearance lies at Themes.
"""

import _tkinter
import sys
import tkinter
from _typeshed import MaybeNone
from collections.abc import Callable, Iterable, Sequence
from tkinter.font import _FontDescription
from typing import Any, Literal, ParamSpec, TypeAlias, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import Never, Unpack

__all__ = [
    "Button",
    "Checkbutton",
    "Combobox",
    "Entry",
    "Frame",
    "Label",
    "Labelframe",
    "LabelFrame",
    "Menubutton",
    "Notebook",
    "Panedwindow",
    "PanedWindow",
    "Progressbar",
    "Radiobutton",
    "Scale",
    "Scrollbar",
    "Separator",
    "Sizegrip",
    "Style",
    "Treeview",
    "LabeledScale",
    "OptionMenu",
    "tclobjs_to_py",
    "setup_master",
    "Spinbox",
]

def tclobjs_to_py(adict: dict[Any, Any]) -> dict[Any, Any]:
    """Returns adict with its values converted from Tcl objects to Python
    objects.
    """

def setup_master(master: tkinter.Misc | None = None):
    """If master is not None, itself is returned. If master is None,
    the default master is returned if there is one, otherwise a new
    master is created and returned.

    If it is not allowed to use the default root and master is None,
    RuntimeError is raised.
    """

_Padding: TypeAlias = (
    float
    | str
    | tuple[float | str]
    | tuple[float | str, float | str]
    | tuple[float | str, float | str, float | str]
    | tuple[float | str, float | str, float | str, float | str]
)

# Last item (option value to apply) varies between different options so use Any.
# It could also be any iterable with items matching the tuple, but that case
# hasn't been added here for consistency with _Padding above.
_Statespec: TypeAlias = tuple[Unpack[tuple[str, ...]], Any]
_ImageStatespec: TypeAlias = tuple[Unpack[tuple[str, ...]], tkinter._Image | str]
_VsapiStatespec: TypeAlias = tuple[Unpack[tuple[str, ...]], int]

_P = ParamSpec("_P")
_T = TypeVar("_T")

@type_check_only
class _Layout(TypedDict, total=False):
    side: Literal["left", "right", "top", "bottom"]
    sticky: str  # consists of letters 'n', 's', 'w', 'e', may contain repeats, may be empty
    unit: Literal[0, 1] | bool
    children: _LayoutSpec
    # Note: there seem to be some other undocumented keys sometimes

# This could be any sequence when passed as a parameter but will always be a list when returned.
_LayoutSpec: TypeAlias = list[tuple[str, _Layout | None]]

# Keep these in sync with the appropriate methods in Style
@type_check_only
class _ElementCreateImageKwargs(TypedDict, total=False):
    border: _Padding
    height: float | str
    padding: _Padding
    sticky: str
    width: float | str

_ElementCreateArgsCrossPlatform: TypeAlias = (
    # Could be any sequence here but types are not homogenous so just type it as tuple
    tuple[Literal["image"], tkinter._Image | str, Unpack[tuple[_ImageStatespec, ...]], _ElementCreateImageKwargs]
    | tuple[Literal["from"], str, str]
    | tuple[Literal["from"], str]  # (fromelement is optional)
)
if sys.platform == "win32" and sys.version_info >= (3, 13):
    @type_check_only
    class _ElementCreateVsapiKwargsPadding(TypedDict, total=False):
        padding: _Padding

    @type_check_only
    class _ElementCreateVsapiKwargsMargin(TypedDict, total=False):
        padding: _Padding

    @type_check_only
    class _ElementCreateVsapiKwargsSize(TypedDict):
        width: float | str
        height: float | str

    _ElementCreateVsapiKwargsDict: TypeAlias = (
        _ElementCreateVsapiKwargsPadding | _ElementCreateVsapiKwargsMargin | _ElementCreateVsapiKwargsSize
    )
    _ElementCreateArgs: TypeAlias = (  # noqa: Y047  # It doesn't recognise the usage below for whatever reason
        _ElementCreateArgsCrossPlatform
        | tuple[Literal["vsapi"], str, int, _ElementCreateVsapiKwargsDict]
        | tuple[Literal["vsapi"], str, int, _VsapiStatespec, _ElementCreateVsapiKwargsDict]
    )
else:
    _ElementCreateArgs: TypeAlias = _ElementCreateArgsCrossPlatform
_ThemeSettingsValue = TypedDict(
    "_ThemeSettingsValue",
    {
        "configure": dict[str, Any],
        "map": dict[str, Iterable[_Statespec]],
        "layout": _LayoutSpec,
        "element create": _ElementCreateArgs,
    },
    total=False,
)
_ThemeSettings: TypeAlias = dict[str, _ThemeSettingsValue]

class Style:
    """Manipulate style database."""

    master: tkinter.Misc
    tk: _tkinter.TkappType
    def __init__(self, master: tkinter.Misc | None = None) -> None: ...

    # For these methods, values given vary between options. Returned values
    # seem to be str, but this might not always be the case.
    @overload
    def configure(self, style: str) -> dict[str, Any] | None:  # Returns None if no configuration.
        """Query or sets the default value of the specified option(s) in
        style.

        Each key in kw is an option and each value is either a string or
        a sequence identifying the value for that option.
        """
    @overload
    def configure(self, style: str, query_opt: str, **kw: Any) -> Any: ...
    @overload
    def configure(self, style: str, query_opt: None = None, **kw: Any) -> None: ...

    @overload
    def map(self, style: str, query_opt: str) -> _Statespec:
        """Query or sets dynamic values of the specified option(s) in
        style.

        Each key in kw is an option and each value should be a list or a
        tuple (usually) containing statespecs grouped in tuples, or list,
        or something else of your preference. A statespec is compound of
        one or more states and then a value.
        """
    @overload
    def map(self, style: str, query_opt: None = None, **kw: Iterable[_Statespec]) -> dict[str, _Statespec]: ...

    def lookup(self, style: str, option: str, state: Iterable[str] | None = None, default: Any | None = None) -> Any:
        """Returns the value specified for option in style.

        If state is specified it is expected to be a sequence of one
        or more states. If the default argument is set, it is used as
        a fallback value in case no specification for option is found.
        """

    @overload
    def layout(self, style: str, layoutspec: _LayoutSpec) -> list[Never]:  # Always seems to return an empty list
        """Define the widget layout for given style. If layoutspec is
        omitted, return the layout specification for given style.

        layoutspec is expected to be a list or an object different than
        None that evaluates to False if you want to "turn off" that style.
        If it is a list (or tuple, or something else), each item should be
        a tuple where the first item is the layout name and the second item
        should have the format described below:

        LAYOUTS

            A layout can contain the value None, if takes no options, or
            a dict of options specifying how to arrange the element.
            The layout mechanism uses a simplified version of the pack
            geometry manager: given an initial cavity, each element is
            allocated a parcel. Valid options/values are:

                side: whichside
                    Specifies which side of the cavity to place the
                    element; one of top, right, bottom or left. If
                    omitted, the element occupies the entire cavity.

                sticky: nswe
                    Specifies where the element is placed inside its
                    allocated parcel.

                children: [sublayout... ]
                    Specifies a list of elements to place inside the
                    element. Each element is a tuple (or other sequence)
                    where the first item is the layout name, and the other
                    is a LAYOUT.
        """
    @overload
    def layout(self, style: str, layoutspec: None = None) -> _LayoutSpec: ...

    @overload
    def element_create(
        self,
        elementname: str,
        etype: Literal["image"],
        default_image: tkinter._Image | str,
        /,
        *imagespec: _ImageStatespec,
        border: _Padding = ...,
        height: float | str = ...,
        padding: _Padding = ...,
        sticky: str = ...,
        width: float | str = ...,
    ) -> None:
        """Create a new element in the current theme of given etype."""
    @overload
    def element_create(self, elementname: str, etype: Literal["from"], themename: str, fromelement: str = ..., /) -> None: ...
    if sys.platform == "win32" and sys.version_info >= (3, 13):  # and tk version >= 8.6
        # margin, padding, and (width + height) are mutually exclusive. width
        # and height must either both be present or not present at all. Note:
        # There are other undocumented options if you look at ttk's source code.
        @overload
        def element_create(
            self,
            elementname: str,
            etype: Literal["vsapi"],
            class_: str,
            part: int,
            vs_statespec: _VsapiStatespec = ...,
            /,
            *,
            padding: _Padding = ...,
        ) -> None:
            """Create a new element in the current theme of given etype."""
        @overload
        def element_create(
            self,
            elementname: str,
            etype: Literal["vsapi"],
            class_: str,
            part: int,
            vs_statespec: _VsapiStatespec = ...,
            /,
            *,
            margin: _Padding = ...,
        ) -> None: ...
        @overload
        def element_create(
            self,
            elementname: str,
            etype: Literal["vsapi"],
            class_: str,
            part: int,
            vs_statespec: _VsapiStatespec = ...,
            /,
            *,
            width: float | str,
            height: float | str,
        ) -> None: ...

    def element_names(self) -> tuple[str, ...]:
        """Returns the list of elements defined in the current theme."""

    def element_options(self, elementname: str) -> tuple[str, ...]:
        """Return the list of elementname's options."""

    def theme_create(self, themename: str, parent: str | None = None, settings: _ThemeSettings | None = None) -> None:
        """Creates a new theme.

        It is an error if themename already exists. If parent is
        specified, the new theme will inherit styles, elements and
        layouts from the specified parent theme. If settings are present,
        they are expected to have the same syntax used for theme_settings.
        """

    def theme_settings(self, themename: str, settings: _ThemeSettings) -> None:
        """Temporarily sets the current theme to themename, apply specified
        settings and then restore the previous theme.

        Each key in settings is a style and each value may contain the
        keys 'configure', 'map', 'layout' and 'element create' and they
        are expected to have the same format as specified by the methods
        configure, map, layout and element_create respectively.
        """

    def theme_names(self) -> tuple[str, ...]:
        """Returns a list of all known themes."""

    @overload
    def theme_use(self, themename: str) -> None:
        """If themename is None, returns the theme in use, otherwise, set
        the current theme to themename, refreshes all widgets and emits
        a <<ThemeChanged>> event.
        """
    @overload
    def theme_use(self, themename: None = None) -> str: ...

class Widget(tkinter.Widget):
    """Base class for Tk themed widgets."""

    def __init__(self, master: tkinter.Misc | None, widgetname: str | None, kw: dict[str, Any] | None = None) -> None:
        """Constructs a Ttk Widget with the parent master.

        STANDARD OPTIONS

            class, cursor, takefocus, style

        SCROLLABLE WIDGET OPTIONS

            xscrollcommand, yscrollcommand

        LABEL WIDGET OPTIONS

            text, textvariable, underline, image, compound, width

        WIDGET STATES

            active, disabled, focus, pressed, selected, background,
            readonly, alternate, invalid
        """

    def identify(self, x: int, y: int) -> str:
        """Returns the name of the element at position x, y, or the empty
        string if the point does not lie within any element.

        x and y are pixel coordinates relative to the widget.
        """

    @overload
    def instate(self, statespec: Sequence[str], callback: None = None) -> bool:
        """Test the widget's state.

        If callback is not specified, returns True if the widget state
        matches statespec and False otherwise. If callback is specified,
        then it will be invoked with *args, **kw if the widget state
        matches statespec. statespec is expected to be a sequence.
        """
    @overload
    def instate(
        self, statespec: Sequence[str], callback: Callable[_P, _T], *args: _P.args, **kw: _P.kwargs
    ) -> Literal[False] | _T: ...

    def state(self, statespec: Sequence[str] | None = None) -> tuple[str, ...]:
        """Modify or inquire widget state.

        Widget state is returned if statespec is None, otherwise it is
        set according to the statespec flags and then a new state spec
        is returned indicating which flags were changed. statespec is
        expected to be a sequence.
        """

class Button(Widget):
    """Ttk Button widget, displays a textual label and/or image, and
    evaluates a command when pressed.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        command: str | Callable[[], Any] = "",
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = "",
        cursor: tkinter._Cursor = "",
        default: Literal["normal", "active", "disabled"] = "normal",
        image: tkinter._Image | str = "",
        name: str = ...,
        padding=...,  # undocumented
        state: str = "normal",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = "",
        textvariable: tkinter.Variable = ...,
        underline: int = -1,
        width: int | Literal[""] = "",
    ) -> None:
        """Construct a Ttk Button widget with the parent master.

        STANDARD OPTIONS

            class, compound, cursor, image, justify (Tk 9.0+), padding,
            state, style, takefocus, text, textvariable, underline, width

        WIDGET-SPECIFIC OPTIONS

            command, default, width
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        command: str | Callable[[], Any] = ...,
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = ...,
        cursor: tkinter._Cursor = ...,
        default: Literal["normal", "active", "disabled"] = ...,
        image: tkinter._Image | str = ...,
        padding=...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = ...,
        textvariable: tkinter.Variable = ...,
        underline: int = ...,
        width: int | Literal[""] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure
    def invoke(self) -> Any:
        """Invokes the command associated with the button."""

class Checkbutton(Widget):
    """Ttk Checkbutton widget which is either in on- or off-state."""

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        command: str | Callable[[], Any] = "",
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = "",
        cursor: tkinter._Cursor = "",
        image: tkinter._Image | str = "",
        name: str = ...,
        offvalue: Any = 0,
        onvalue: Any = 1,
        padding=...,  # undocumented
        state: str = "normal",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = "",
        textvariable: tkinter.Variable = ...,
        underline: int = -1,
        # Seems like variable can be empty string, but actually setting it to
        # empty string segfaults before Tcl 8.6.9. Search for ttk::checkbutton
        # here: https://sourceforge.net/projects/tcl/files/Tcl/8.6.9/tcltk-release-notes-8.6.9.txt/view
        variable: tkinter.Variable = ...,
        width: int | Literal[""] = "",
    ) -> None:
        """Construct a Ttk Checkbutton widget with the parent master.

        STANDARD OPTIONS

            class, compound, cursor, image, justify (Tk 9.0+), padding,
            state, style, takefocus, text, textvariable, underline, width

        WIDGET-SPECIFIC OPTIONS

            command, offvalue, onvalue, variable
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        command: str | Callable[[], Any] = ...,
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = ...,
        cursor: tkinter._Cursor = ...,
        image: tkinter._Image | str = ...,
        offvalue: Any = ...,
        onvalue: Any = ...,
        padding=...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = ...,
        textvariable: tkinter.Variable = ...,
        underline: int = ...,
        variable: tkinter.Variable = ...,
        width: int | Literal[""] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure
    def invoke(self) -> Any:
        """Toggles between the selected and deselected states and
        invokes the associated command. If the widget is currently
        selected, sets the option variable to the offvalue option
        and deselects the widget; otherwise, sets the option variable
        to the option onvalue.

        Returns the result of the associated command.
        """

class Entry(Widget, tkinter.Entry):
    """Ttk Entry widget displays a one-line text string and allows that
    string to be edited by the user.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        widget: str | None = None,
        *,
        background: str = ...,  # undocumented
        class_: str = "",
        cursor: tkinter._Cursor = ...,
        exportselection: bool = True,
        font: _FontDescription = "TkTextFont",
        foreground: str = "",
        invalidcommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = "",
        justify: Literal["left", "center", "right"] = "left",
        name: str = ...,
        show: str = "",
        state: str = "normal",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        textvariable: tkinter.Variable = ...,
        validate: Literal["none", "focus", "focusin", "focusout", "key", "all"] = "none",
        validatecommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = "",
        width: int = 20,
        xscrollcommand: str | Callable[[float, float], object] = "",
    ) -> None:
        """Constructs a Ttk Entry widget with the parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus, xscrollcommand

        WIDGET-SPECIFIC OPTIONS

            background, exportselection, font, foreground, invalidcommand,
            justify, locale (Tk 9.1+), placeholder (Tk 9.0+),
            placeholderforeground (Tk 9.0+), show, state, textvariable,
            validate, validatecommand, width

        VALIDATION MODES

            none, key, focus, focusin, focusout, all
        """

    @overload  # type: ignore[override]
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        background: str = ...,
        cursor: tkinter._Cursor = ...,
        exportselection: bool = ...,
        font: _FontDescription = ...,
        foreground: str = ...,
        invalidcommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        justify: Literal["left", "center", "right"] = ...,
        show: str = ...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        textvariable: tkinter.Variable = ...,
        validate: Literal["none", "focus", "focusin", "focusout", "key", "all"] = ...,
        validatecommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        width: int = ...,
        xscrollcommand: str | Callable[[float, float], object] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    # config must be copy/pasted, otherwise ttk.Entry().config is mypy error (don't know why)
    @overload  # type: ignore[override]
    def config(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        background: str = ...,
        cursor: tkinter._Cursor = ...,
        exportselection: bool = ...,
        font: _FontDescription = ...,
        foreground: str = ...,
        invalidcommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        justify: Literal["left", "center", "right"] = ...,
        show: str = ...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        textvariable: tkinter.Variable = ...,
        validate: Literal["none", "focus", "focusin", "focusout", "key", "all"] = ...,
        validatecommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        width: int = ...,
        xscrollcommand: str | Callable[[float, float], object] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def config(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    def bbox(self, index) -> tuple[int, int, int, int]:  # type: ignore[override]
        """Return a tuple of (x, y, width, height) which describes the
        bounding box of the character given by index.
        """

    def identify(self, x: int, y: int) -> str:
        """Returns the name of the element at position x, y, or the
        empty string if the coordinates are outside the window.
        """

    def validate(self):
        """Force revalidation, independent of the conditions specified
        by the validate option. Returns False if validation fails, True
        if it succeeds. Sets or clears the invalid state accordingly.
        """

class Combobox(Entry):
    """Ttk Combobox widget combines a text field with a pop-down list of
    values.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        background: str = ...,  # undocumented
        class_: str = "",
        cursor: tkinter._Cursor = "",
        exportselection: bool = True,
        font: _FontDescription = ...,  # undocumented
        foreground: str = ...,  # undocumented
        height: int = 10,
        invalidcommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,  # undocumented
        justify: Literal["left", "center", "right"] = "left",
        name: str = ...,
        postcommand: Callable[[], object] | str = "",
        show=...,  # undocumented
        state: str = "normal",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        textvariable: tkinter.Variable = ...,
        validate: Literal["none", "focus", "focusin", "focusout", "key", "all"] = ...,  # undocumented
        validatecommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,  # undocumented
        values: list[str] | tuple[str, ...] = ...,
        width: int = 20,
        xscrollcommand: str | Callable[[float, float], object] = ...,  # undocumented
    ) -> None:
        """Construct a Ttk Combobox widget with the parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus, xscrollcommand

        WIDGET-SPECIFIC OPTIONS

            background, exportselection, font, foreground, height,
            invalidcommand, justify, locale (Tk 9.1+), placeholder (Tk 9.0+),
            placeholderforeground (Tk 9.0+), postcommand, show, state,
            textvariable, validate, validatecommand, values, width
        """

    @overload  # type: ignore[override]
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        background: str = ...,
        cursor: tkinter._Cursor = ...,
        exportselection: bool = ...,
        font: _FontDescription = ...,
        foreground: str = ...,
        height: int = ...,
        invalidcommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        justify: Literal["left", "center", "right"] = ...,
        postcommand: Callable[[], object] | str = ...,
        show=...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        textvariable: tkinter.Variable = ...,
        validate: Literal["none", "focus", "focusin", "focusout", "key", "all"] = ...,
        validatecommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        values: list[str] | tuple[str, ...] = ...,
        width: int = ...,
        xscrollcommand: str | Callable[[float, float], object] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    # config must be copy/pasted, otherwise ttk.Combobox().config is mypy error (don't know why)
    @overload  # type: ignore[override]
    def config(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        background: str = ...,
        cursor: tkinter._Cursor = ...,
        exportselection: bool = ...,
        font: _FontDescription = ...,
        foreground: str = ...,
        height: int = ...,
        invalidcommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        justify: Literal["left", "center", "right"] = ...,
        postcommand: Callable[[], object] | str = ...,
        show=...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        textvariable: tkinter.Variable = ...,
        validate: Literal["none", "focus", "focusin", "focusout", "key", "all"] = ...,
        validatecommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        values: list[str] | tuple[str, ...] = ...,
        width: int = ...,
        xscrollcommand: str | Callable[[float, float], object] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def config(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    def current(self, newindex: int | None = None) -> int:
        """If newindex is supplied, sets the combobox value to the
        element at position newindex in the list of values. Otherwise,
        returns the index of the current value in the list of values
        or -1 if the current value does not appear in the list.
        """

    def set(self, value: Any) -> None:
        """Sets the value of the combobox to value."""

class Frame(Widget):
    """Ttk Frame widget is a container, used to group other widgets
    together.
    """

    # This should be kept in sync with tkinter.ttk.LabeledScale.__init__()
    # (all of these keyword-only arguments are also present there)
    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        border: float | str = ...,
        borderwidth: float | str = ...,
        class_: str = "",
        cursor: tkinter._Cursor = "",
        height: float | str = 0,
        name: str = ...,
        padding: _Padding = ...,
        relief: Literal["raised", "sunken", "flat", "ridge", "solid", "groove"] = ...,
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = "",
        width: float | str = 0,
    ) -> None:
        """Construct a Ttk Frame with parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus

        WIDGET-SPECIFIC OPTIONS

            borderwidth, relief, padding, width, height
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        border: float | str = ...,
        borderwidth: float | str = ...,
        cursor: tkinter._Cursor = ...,
        height: float | str = ...,
        padding: _Padding = ...,
        relief: Literal["raised", "sunken", "flat", "ridge", "solid", "groove"] = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        width: float | str = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure

class Label(Widget):
    """Ttk Label widget displays a textual label and/or image."""

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        anchor: Literal["nw", "n", "ne", "w", "center", "e", "sw", "s", "se"] = ...,
        background: str = "",
        border: float | str = ...,  # alias for borderwidth
        borderwidth: float | str = ...,  # undocumented
        class_: str = "",
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = "",
        cursor: tkinter._Cursor = "",
        font: _FontDescription = ...,
        foreground: str = "",
        image: tkinter._Image | str = "",
        justify: Literal["left", "center", "right"] = ...,
        name: str = ...,
        padding: _Padding = ...,
        relief: Literal["raised", "sunken", "flat", "ridge", "solid", "groove"] = ...,
        state: str = "normal",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = "",
        text: float | str = "",
        textvariable: tkinter.Variable = ...,
        underline: int = -1,
        width: int | Literal[""] = "",
        wraplength: float | str = ...,
    ) -> None:
        """Construct a Ttk Label with parent master.

        STANDARD OPTIONS

            class, compound, cursor, image, state, style, takefocus,
            text, textvariable, underline, width

        WIDGET-SPECIFIC OPTIONS

            anchor, background, borderwidth, font, foreground, justify,
            padding, relief, text, textangle (Tk 9.1+), wraplength
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        anchor: Literal["nw", "n", "ne", "w", "center", "e", "sw", "s", "se"] = ...,
        background: str = ...,
        border: float | str = ...,
        borderwidth: float | str = ...,
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = ...,
        cursor: tkinter._Cursor = ...,
        font: _FontDescription = ...,
        foreground: str = ...,
        image: tkinter._Image | str = ...,
        justify: Literal["left", "center", "right"] = ...,
        padding: _Padding = ...,
        relief: Literal["raised", "sunken", "flat", "ridge", "solid", "groove"] = ...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = ...,
        textvariable: tkinter.Variable = ...,
        underline: int = ...,
        width: int | Literal[""] = ...,
        wraplength: float | str = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure

class Labelframe(Widget):
    """Ttk Labelframe widget is a container used to group other widgets
    together. It has an optional label, which may be a plain text string
    or another widget.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        border: float | str = ...,
        borderwidth: float | str = ...,  # undocumented
        class_: str = "",
        cursor: tkinter._Cursor = "",
        height: float | str = 0,
        labelanchor: Literal["nw", "n", "ne", "en", "e", "es", "se", "s", "sw", "ws", "w", "wn"] = ...,
        labelwidget: tkinter.Misc = ...,
        name: str = ...,
        padding: _Padding = ...,
        relief: Literal["raised", "sunken", "flat", "ridge", "solid", "groove"] = ...,  # undocumented
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = "",
        text: float | str = "",
        underline: int = -1,
        width: float | str = 0,
    ) -> None:
        """Construct a Ttk Labelframe with parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus

        WIDGET-SPECIFIC OPTIONS

            borderwidth, height, labelanchor, labelwidget, padding,
            relief, text, underline, width
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        border: float | str = ...,
        borderwidth: float | str = ...,
        cursor: tkinter._Cursor = ...,
        height: float | str = ...,
        labelanchor: Literal["nw", "n", "ne", "en", "e", "es", "se", "s", "sw", "ws", "w", "wn"] = ...,
        labelwidget: tkinter.Misc = ...,
        padding: _Padding = ...,
        relief: Literal["raised", "sunken", "flat", "ridge", "solid", "groove"] = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = ...,
        underline: int = ...,
        width: float | str = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure

LabelFrame = Labelframe

class Menubutton(Widget):
    """Ttk Menubutton widget displays a textual label and/or image, and
    displays a menu when pressed.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = "",
        cursor: tkinter._Cursor = "",
        direction: Literal["above", "below", "left", "right", "flush"] = "below",
        image: tkinter._Image | str = "",
        menu: tkinter.Menu = ...,
        name: str = ...,
        padding=...,  # undocumented
        state: str = "normal",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = "",
        textvariable: tkinter.Variable = ...,
        underline: int = -1,
        width: int | Literal[""] = "",
    ) -> None:
        """Construct a Ttk Menubutton with parent master.

        STANDARD OPTIONS

            class, compound, cursor, image, justify (Tk 9.0+), padding,
            state, style, takefocus, text, textvariable, underline, width

        WIDGET-SPECIFIC OPTIONS

            direction, menu
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = ...,
        cursor: tkinter._Cursor = ...,
        direction: Literal["above", "below", "left", "right", "flush"] = ...,
        image: tkinter._Image | str = ...,
        menu: tkinter.Menu = ...,
        padding=...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = ...,
        textvariable: tkinter.Variable = ...,
        underline: int = ...,
        width: int | Literal[""] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure

class Notebook(Widget):
    """Ttk Notebook widget manages a collection of windows and displays
    a single one at a time. Each child window is associated with a tab,
    which the user may select to change the currently-displayed window.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        cursor: tkinter._Cursor = "",
        height: int = 0,
        name: str = ...,
        padding: _Padding = ...,
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        width: int = 0,
    ) -> None:
        """Construct a Ttk Notebook with parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus

        WIDGET-SPECIFIC OPTIONS

            height, padding, width

        TAB OPTIONS

            state, sticky, padding, text, image, compound, underline

        TAB IDENTIFIERS (tab_id)

            The tab_id argument found in several methods may take any of
            the following forms:

                * An integer between zero and the number of tabs
                * The name of a child window
                * A positional specification of the form "@x,y", which
                  defines the tab
                * The string "current", which identifies the
                  currently-selected tab
                * The string "end", which returns the number of tabs (only
                  valid for method index)
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        cursor: tkinter._Cursor = ...,
        height: int = ...,
        padding: _Padding = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        width: int = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure
    def add(
        self,
        child: tkinter.Widget,
        *,
        state: Literal["normal", "disabled", "hidden"] = ...,
        sticky: str = ...,  # consists of letters 'n', 's', 'w', 'e', no repeats, may be empty
        padding: _Padding = ...,
        text: str = ...,
        # `image` is a sequence of an image name, followed by zero or more
        # (sequences of one or more state names followed by an image name)
        image=...,
        compound: Literal["top", "left", "center", "right", "bottom", "none"] = ...,
        underline: int = ...,
    ) -> None:
        """Adds a new tab to the notebook.

        If window is currently managed by the notebook but hidden, it is
        restored to its previous position.
        """

    def forget(self, tab_id) -> None:  # type: ignore[override]
        """Removes the tab specified by tab_id, unmaps and unmanages the
        associated window.
        """

    def hide(self, tab_id) -> None:
        """Hides the tab specified by tab_id.

        The tab will not be displayed, but the associated window remains
        managed by the notebook and its configuration remembered. Hidden
        tabs may be restored with the add command.
        """

    def identify(self, x: int, y: int) -> str:
        """Returns the name of the tab element at position x, y, or the
        empty string if none.
        """

    def index(self, tab_id):
        """Returns the numeric index of the tab specified by tab_id, or
        the total number of tabs if tab_id is the string "end".
        """

    def insert(self, pos, child, **kw) -> None:
        """Inserts a pane at the specified position.

        pos is either the string end, an integer index, or the name of
        a managed child. If child is already managed by the notebook,
        moves it to the specified position.
        """

    def select(self, tab_id=None):
        """Selects the specified tab.

        The associated child window will be displayed, and the
        previously-selected window (if different) is unmapped. If tab_id
        is omitted, returns the widget name of the currently selected
        pane.
        """

    def tab(self, tab_id, option=None, **kw):
        """Query or modify the options of the specific tab_id.

        If kw is not given, returns a dict of the tab option values. If option
        is specified, returns the value of that option. Otherwise, sets the
        options to the corresponding values.
        """

    def tabs(self):
        """Returns a list of windows managed by the notebook."""

    def enable_traversal(self) -> None:
        """Enable keyboard traversal for a toplevel window containing
        this notebook.

        This will extend the bindings for the toplevel window containing
        this notebook as follows:

            Control-Tab: selects the tab following the currently selected
                         one

            Shift-Control-Tab: selects the tab preceding the currently
                               selected one

            Alt-K: where K is the mnemonic (underlined) character of any
                   tab, will select that tab.

        Multiple notebooks in a single toplevel may be enabled for
        traversal, including nested notebooks. However, notebook traversal
        only works properly if all panes are direct children of the
        notebook.
        """

class Panedwindow(Widget, tkinter.PanedWindow):
    """Ttk Panedwindow widget displays a number of subwindows, stacked
    either vertically or horizontally.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        cursor: tkinter._Cursor = "",
        # width and height for tkinter.ttk.Panedwindow are int but for tkinter.PanedWindow they are screen units
        height: int = 0,
        name: str = ...,
        orient: Literal["vertical", "horizontal"] = "vertical",  # can't be changed with configure()
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = "",
        width: int = 0,
    ) -> None:
        """Construct a Ttk Panedwindow with parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus

        WIDGET-SPECIFIC OPTIONS

            orient, width, height

        PANE OPTIONS

            weight
        """

    def add(self, child: tkinter.Widget, *, weight: int = ..., **kw) -> None:
        """Add a child widget to the panedwindow in a new pane.

        The child argument is the name of the child widget
        followed by pairs of arguments that specify how to
        manage the windows. The possible options and values
        are the ones accepted by the paneconfigure method.
        """

    @overload  # type: ignore[override]
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        cursor: tkinter._Cursor = ...,
        height: int = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        width: int = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    # config must be copy/pasted, otherwise ttk.Panedwindow().config is mypy error (don't know why)
    @overload  # type: ignore[override]
    def config(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        cursor: tkinter._Cursor = ...,
        height: int = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        width: int = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def config(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    forget = tkinter.PanedWindow.forget
    def insert(self, pos, child, **kw) -> None:
        """Inserts a pane at the specified positions.

        pos is either the string end, and integer index, or the name
        of a child. If child is already managed by the paned window,
        moves it to the specified position.
        """

    def pane(self, pane, option=None, **kw):
        """Query or modify the options of the specified pane.

        pane is either an integer index or the name of a managed subwindow.
        If kw is not given, returns a dict of the pane option values. If
        option is specified then the value for that option is returned.
        Otherwise, sets the options to the corresponding values.
        """

    def sashpos(self, index, newpos=None):
        """If newpos is specified, sets the position of sash number index.

        May adjust the positions of adjacent sashes to ensure that
        positions are monotonically increasing. Sash positions are further
        constrained to be between 0 and the total size of the widget.

        Returns the new position of sash number index.
        """

PanedWindow = Panedwindow

class Progressbar(Widget):
    """Ttk Progressbar widget shows the status of a long-running
    operation. They can operate in two modes: determinate mode shows the
    amount completed relative to the total amount of work to be done, and
    indeterminate mode provides an animated display to let the user know
    that something is happening.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        cursor: tkinter._Cursor = "",
        length: float | str = 100,
        maximum: float = 100,
        mode: Literal["determinate", "indeterminate"] = "determinate",
        name: str = ...,
        orient: Literal["horizontal", "vertical"] = "horizontal",
        phase: int = 0,  # docs say read-only but assigning int to this works
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = "",
        value: float = 0.0,
        variable: tkinter.IntVar | tkinter.DoubleVar = ...,
    ) -> None:
        """Construct a Ttk Progressbar with parent master.

        STANDARD OPTIONS

            anchor (Tk 9.0+), class, cursor, font (Tk 9.0+),
            foreground (Tk 9.0+), justify (Tk 9.0+), style, takefocus,
            text (Tk 9.0+), wraplength (Tk 9.0+)

        WIDGET-SPECIFIC OPTIONS

            orient, length, mode, maximum, value, variable, phase
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        cursor: tkinter._Cursor = ...,
        length: float | str = ...,
        maximum: float = ...,
        mode: Literal["determinate", "indeterminate"] = ...,
        orient: Literal["horizontal", "vertical"] = ...,
        phase: int = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        value: float = ...,
        variable: tkinter.IntVar | tkinter.DoubleVar = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure
    def start(self, interval: Literal["idle"] | int | None = None) -> None:
        """Begin autoincrement mode: schedules a recurring timer event
        that calls method step every interval milliseconds.

        interval defaults to 50 milliseconds (20 steps/second) if omitted.
        """

    def step(self, amount: float | None = None) -> None:
        """Increments the value option by amount.

        amount defaults to 1.0 if omitted.
        """

    def stop(self) -> None:
        """Stop autoincrement mode: cancels any recurring timer event
        initiated by start.
        """

class Radiobutton(Widget):
    """Ttk Radiobutton widgets are used in groups to show or change a
    set of mutually-exclusive options.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        command: str | Callable[[], Any] = "",
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = "",
        cursor: tkinter._Cursor = "",
        image: tkinter._Image | str = "",
        name: str = ...,
        padding=...,  # undocumented
        state: str = "normal",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = "",
        textvariable: tkinter.Variable = ...,
        underline: int = -1,
        value: Any = "1",
        variable: tkinter.Variable | Literal[""] = ...,
        width: int | Literal[""] = "",
    ) -> None:
        """Construct a Ttk Radiobutton with parent master.

        STANDARD OPTIONS

            class, compound, cursor, image, justify (Tk 9.0+), padding,
            state, style, takefocus, text, textvariable, underline, width

        WIDGET-SPECIFIC OPTIONS

            command, value, variable
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        command: str | Callable[[], Any] = ...,
        compound: Literal["", "text", "image", "top", "left", "center", "right", "bottom", "none"] = ...,
        cursor: tkinter._Cursor = ...,
        image: tkinter._Image | str = ...,
        padding=...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        text: float | str = ...,
        textvariable: tkinter.Variable = ...,
        underline: int = ...,
        value: Any = ...,
        variable: tkinter.Variable | Literal[""] = ...,
        width: int | Literal[""] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure
    def invoke(self) -> Any:
        """Sets the option variable to the option value, selects the
        widget, and invokes the associated command.

        Returns the result of the command, or an empty string if
        no command is specified.
        """

# type ignore, because identify() methods of Widget and tkinter.Scale are incompatible
class Scale(Widget, tkinter.Scale):  # type: ignore[misc]
    """Ttk Scale widget is typically used to control the numeric value of
    a linked variable that varies uniformly over some range.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        command: str | Callable[[str], object] = "",
        cursor: tkinter._Cursor = "",
        from_: float = 0,
        length: float | str = 100,
        name: str = ...,
        orient: Literal["horizontal", "vertical"] = "horizontal",
        state: str = ...,  # undocumented
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        to: float = 1.0,
        value: float = 0,
        variable: tkinter.IntVar | tkinter.DoubleVar = ...,
    ) -> None:
        """Construct a Ttk Scale with parent master.

        STANDARD OPTIONS

            class, cursor, state, style, takefocus

        WIDGET-SPECIFIC OPTIONS

            command, from, length, orient, to, value, variable
        """

    @overload  # type: ignore[override]
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        command: str | Callable[[str], object] = ...,
        cursor: tkinter._Cursor = ...,
        from_: float = ...,
        length: float | str = ...,
        orient: Literal["horizontal", "vertical"] = ...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        to: float = ...,
        value: float = ...,
        variable: tkinter.IntVar | tkinter.DoubleVar = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Modify or query scale options.

        Setting a value for any of the "from", "from_" or "to" options
        generates a <<RangeChanged>> event.
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    # config must be copy/pasted, otherwise ttk.Scale().config is mypy error (don't know why)
    @overload  # type: ignore[override]
    def config(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        command: str | Callable[[str], object] = ...,
        cursor: tkinter._Cursor = ...,
        from_: float = ...,
        length: float | str = ...,
        orient: Literal["horizontal", "vertical"] = ...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        to: float = ...,
        value: float = ...,
        variable: tkinter.IntVar | tkinter.DoubleVar = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def config(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    def get(self, x: int | None = None, y: int | None = None) -> float:
        """Get the current value of the value option, or the value
        corresponding to the coordinates x, y if they are specified.

        x and y are pixel coordinates relative to the scale widget
        origin.
        """

# type ignore, because identify() methods of Widget and tkinter.Scale are incompatible
class Scrollbar(Widget, tkinter.Scrollbar):  # type: ignore[misc]
    """Ttk Scrollbar controls the viewport of a scrollable widget."""

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        command: Callable[..., tuple[float, float] | None] | str = "",
        cursor: tkinter._Cursor = "",
        name: str = ...,
        orient: Literal["horizontal", "vertical"] = "vertical",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = "",
    ) -> None:
        """Construct a Ttk Scrollbar with parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus

        WIDGET-SPECIFIC OPTIONS

            command, orient
        """

    @overload  # type: ignore[override]
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        command: Callable[..., tuple[float, float] | None] | str = ...,
        cursor: tkinter._Cursor = ...,
        orient: Literal["horizontal", "vertical"] = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    # config must be copy/pasted, otherwise ttk.Scrollbar().config is mypy error (don't know why)
    @overload  # type: ignore[override]
    def config(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        command: Callable[..., tuple[float, float] | None] | str = ...,
        cursor: tkinter._Cursor = ...,
        orient: Literal["horizontal", "vertical"] = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def config(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

class Separator(Widget):
    """Ttk Separator widget displays a horizontal or vertical separator
    bar.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        cursor: tkinter._Cursor = "",
        name: str = ...,
        orient: Literal["horizontal", "vertical"] = "horizontal",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = "",
    ) -> None:
        """Construct a Ttk Separator with parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus

        WIDGET-SPECIFIC OPTIONS

            orient
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        cursor: tkinter._Cursor = ...,
        orient: Literal["horizontal", "vertical"] = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure

class Sizegrip(Widget):
    """Ttk Sizegrip allows the user to resize the containing toplevel
    window by pressing and dragging the grip.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        cursor: tkinter._Cursor = ...,
        name: str = ...,
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = "",
    ) -> None:
        """Construct a Ttk Sizegrip with parent master.

        STANDARD OPTIONS

            class, cursor, state, style, takefocus
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        cursor: tkinter._Cursor = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure

class Spinbox(Entry):
    """Ttk Spinbox is an Entry with increment and decrement arrows

    It is commonly used for number entry or to select from a list of
    string values.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        background: str = ...,  # undocumented
        class_: str = "",
        command: Callable[[], object] | str | list[str] | tuple[str, ...] = "",
        cursor: tkinter._Cursor = "",
        exportselection: bool = ...,  # undocumented
        font: _FontDescription = ...,  # undocumented
        foreground: str = ...,  # undocumented
        format: str = "",
        from_: float = 0,
        increment: float = 1,
        invalidcommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,  # undocumented
        justify: Literal["left", "center", "right"] = ...,  # undocumented
        name: str = ...,
        show=...,  # undocumented
        state: str = "normal",
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        textvariable: tkinter.Variable = ...,  # undocumented
        to: float = 0,
        validate: Literal["none", "focus", "focusin", "focusout", "key", "all"] = "none",
        validatecommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = "",
        values: list[str] | tuple[str, ...] = ...,
        width: int = ...,  # undocumented
        wrap: bool = False,
        xscrollcommand: str | Callable[[float, float], object] = "",
    ) -> None:
        """Construct a Ttk Spinbox widget with the parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus, validate,
            validatecommand, xscrollcommand, invalidcommand

        WIDGET-SPECIFIC OPTIONS

            background, command, exportselection, font, foreground,
            format, from_, increment, justify, locale (Tk 9.1+),
            placeholder (Tk 9.0+), placeholderforeground (Tk 9.0+), show,
            state, textvariable, to, values, width, wrap
        """

    @overload  # type: ignore[override]
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        background: str = ...,
        command: Callable[[], object] | str | list[str] | tuple[str, ...] = ...,
        cursor: tkinter._Cursor = ...,
        exportselection: bool = ...,
        font: _FontDescription = ...,
        foreground: str = ...,
        format: str = ...,
        from_: float = ...,
        increment: float = ...,
        invalidcommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        justify: Literal["left", "center", "right"] = ...,
        show=...,
        state: str = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        textvariable: tkinter.Variable = ...,
        to: float = ...,
        validate: Literal["none", "focus", "focusin", "focusout", "key", "all"] = ...,
        validatecommand: str | list[str] | tuple[str, ...] | Callable[[], bool] = ...,
        values: list[str] | tuple[str, ...] = ...,
        width: int = ...,
        wrap: bool = ...,
        xscrollcommand: str | Callable[[float, float], object] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure  # type: ignore[assignment]
    def set(self, value: Any) -> None:
        """Sets the value of the Spinbox to value."""

@type_check_only
class _TreeviewItemDict(TypedDict):
    text: str
    image: list[str] | Literal[""]  # no idea why it's wrapped in list
    values: list[Any] | Literal[""]
    open: bool  # actually 0 or 1
    tags: list[str] | Literal[""]

@type_check_only
class _TreeviewTagDict(TypedDict):
    # There is also 'text' and 'anchor', but they don't seem to do anything, using them is likely a bug
    foreground: str
    background: str
    font: _FontDescription
    image: str  # not wrapped in list :D

@type_check_only
class _TreeviewHeaderDict(TypedDict):
    text: str
    image: list[str] | Literal[""]
    anchor: Literal["nw", "n", "ne", "w", "center", "e", "sw", "s", "se"]
    command: str
    state: str  # Doesn't seem to appear anywhere else than in these dicts

@type_check_only
class _TreeviewColumnDict(TypedDict):
    width: int
    minwidth: int
    stretch: bool  # actually 0 or 1
    anchor: Literal["nw", "n", "ne", "w", "center", "e", "sw", "s", "se"]
    id: str

class Treeview(Widget, tkinter.XView, tkinter.YView):
    """Ttk Treeview widget displays a hierarchical collection of items.

    Each item has a textual label, an optional image, and an optional list
    of data values. The data values are displayed in successive columns
    after the tree label.
    """

    def __init__(
        self,
        master: tkinter.Misc | None = None,
        *,
        class_: str = "",
        columns: str | list[str] | list[int] | list[str | int] | tuple[str | int, ...] = "",
        cursor: tkinter._Cursor = "",
        displaycolumns: str | int | list[str] | tuple[str, ...] | list[int] | tuple[int, ...] = ("#all",),
        height: int = 10,
        name: str = ...,
        padding: _Padding = ...,
        selectmode: Literal["extended", "browse", "none"] = "extended",
        # list/tuple of Literal don't actually work in mypy
        #
        # 'tree headings' is same as ['tree', 'headings'], and I wouldn't be
        # surprised if someone is using it.
        show: Literal["tree", "headings", "tree headings", ""] | list[str] | tuple[str, ...] = ("tree", "headings"),
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        xscrollcommand: str | Callable[[float, float], object] = "",
        yscrollcommand: str | Callable[[float, float], object] = "",
    ) -> None:
        """Construct a Ttk Treeview with parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus, xscrollcommand,
            yscrollcommand

        WIDGET-SPECIFIC OPTIONS

            columns, displaycolumns, headingheight (Tk 9.1+), height,
            padding, rowheight (Tk 9.1+), selectmode, selecttype (Tk 9.0+),
            show, striped (Tk 9.0+), titlecolumns (Tk 9.0+),
            titleitems (Tk 9.0+)

        ITEM OPTIONS

            text, image, values, open, tags

        TAG OPTIONS

            foreground, background, font, image
        """

    @overload
    def configure(
        self,
        cnf: dict[str, Any] | None = None,
        *,
        columns: str | list[str] | list[int] | list[str | int] | tuple[str | int, ...] = ...,
        cursor: tkinter._Cursor = ...,
        displaycolumns: str | int | list[str] | tuple[str, ...] | list[int] | tuple[int, ...] = ...,
        height: int = ...,
        padding: _Padding = ...,
        selectmode: Literal["extended", "browse", "none"] = ...,
        show: Literal["tree", "headings", "tree headings", ""] | list[str] | tuple[str, ...] = ...,
        style: str = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        xscrollcommand: str | Callable[[float, float], object] = ...,
        yscrollcommand: str | Callable[[float, float], object] = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None:
        """Query or modify the configuration options of the widget.

        If no arguments are specified, return a dictionary describing
        all of the available options for the widget.

        If an option name is specified, then return a tuple describing
        the one named option.

        If one or more keyword arguments are specified or a dictionary
        is specified, then modify the widget option(s) to have the given
        value(s).
        """
    @overload
    def configure(self, cnf: str) -> tuple[str, str, str, Any, Any]: ...

    config = configure
    def bbox(self, item: str | int, column: str | int | None = None) -> tuple[int, int, int, int] | Literal[""]:  # type: ignore[override]
        """Returns the bounding box (relative to the treeview widget's
        window) of the specified item in the form x y width height.

        If column is specified, returns the bounding box of that cell.
        If the item is not visible (i.e., if it is a descendant of a
        closed item or is scrolled offscreen), returns an empty string.
        """

    def get_children(self, item: str | int | None = None) -> tuple[str, ...]:
        """Returns a tuple of children belonging to item.

        If item is not specified, returns root children.
        """

    def set_children(self, item: str | int, *newchildren: str | int) -> None:
        """Replaces item's child with newchildren.

        Children present in item that are not present in newchildren
        are detached from tree. No items in newchildren may be an
        ancestor of item.
        """

    @overload
    def column(self, column: str | int, option: Literal["width", "minwidth"]) -> int:
        """Query or modify the options for the specified column.

        If kw is not given, returns a dict of the column option values. If
        option is specified then the value for that option is returned.
        Otherwise, sets the options to the corresponding values.
        """
    @overload
    def column(self, column: str | int, option: Literal["stretch"]) -> bool: ...  # actually 0 or 1
    @overload
    def column(self, column: str | int, option: Literal["anchor"]) -> _tkinter.Tcl_Obj: ...
    @overload
    def column(self, column: str | int, option: Literal["id"]) -> str: ...
    @overload
    def column(self, column: str | int, option: str) -> Any: ...
    @overload
    def column(
        self,
        column: str | int,
        option: None = None,
        *,
        width: int = ...,
        minwidth: int = ...,
        stretch: bool = ...,
        anchor: Literal["nw", "n", "ne", "w", "center", "e", "sw", "s", "se"] = ...,
        # id is read-only
    ) -> _TreeviewColumnDict | None: ...

    def delete(self, *items: str | int) -> None:
        """Delete all specified items and all their descendants. The root
        item may not be deleted.
        """

    def detach(self, *items: str | int) -> None:
        """Unlinks all of the specified items from the tree.

        The items and all of their descendants are still present, and may
        be reinserted at another point in the tree, but will not be
        displayed. The root item may not be detached.
        """

    def exists(self, item: str | int) -> bool:
        """Returns True if the specified item is present in the tree,
        False otherwise.
        """

    @overload  # type: ignore[override]
    def focus(self, item: None = None) -> str:  # can return empty string
        """If item is specified, sets the focus item to item. Otherwise,
        returns the current focus item, or '' if there is none.
        """
    @overload
    def focus(self, item: str | int) -> Literal[""]: ...

    @overload
    def heading(self, column: str | int, option: Literal["text"]) -> str:
        """Query or modify the heading options for the specified column.

        If kw is not given, returns a dict of the heading option values. If
        option is specified then the value for that option is returned.
        Otherwise, sets the options to the corresponding values.

        Valid options/values are:
            text: text
                The text to display in the column heading
            image: image_name
                Specifies an image to display to the right of the column
                heading
            anchor: anchor
                Specifies how the heading text should be aligned. One of
                the standard Tk anchor values
            command: callback
                A callback to be invoked when the heading label is
                pressed.

        To configure the tree column heading, call this with column = "#0"
        """
    @overload
    def heading(self, column: str | int, option: Literal["image"]) -> tuple[str] | str: ...
    @overload
    def heading(self, column: str | int, option: Literal["anchor"]) -> _tkinter.Tcl_Obj: ...
    @overload
    def heading(self, column: str | int, option: Literal["command"]) -> str: ...
    @overload
    def heading(self, column: str | int, option: str) -> Any: ...
    @overload
    def heading(self, column: str | int, option: None = None) -> _TreeviewHeaderDict: ...
    @overload
    def heading(
        self,
        column: str | int,
        option: None = None,
        *,
        text: str = ...,
        image: tkinter._Image | str = ...,
        anchor: Literal["nw", "n", "ne", "w", "center", "e", "sw", "s", "se"] = ...,
        command: str | Callable[[], object] = ...,
    ) -> None: ...

    # Internal Method. Leave untyped:
    def identify(self, component, x, y):  # type: ignore[override]
        """Returns a description of the specified component under the
        point given by x and y, or the empty string if no such component
        is present at that position.
        """

    def identify_row(self, y: int) -> str:
        """Returns the item ID of the item at position y."""

    def identify_column(self, x: int) -> str:
        """Returns the data column identifier of the cell at position x.

        The tree column has ID #0.
        """

    def identify_region(self, x: int, y: int) -> Literal["heading", "separator", "tree", "cell", "nothing"]:
        """Returns one of:

        heading: Tree heading area.
        separator: Space between two columns headings;
        tree: The tree area.
        cell: A data cell.

        * Availability: Tk 8.6
        """

    def identify_element(self, x: int, y: int) -> str:  # don't know what possible return values are
        """Returns the element at position x, y.

        * Availability: Tk 8.6
        """

    def index(self, item: str | int) -> int:
        """Returns the integer index of item within its parent's list
        of children.
        """

    def insert(
        self,
        parent: str,
        index: int | Literal["end"],
        iid: str | int | None = None,
        *,
        id: str | int = ...,  # same as iid
        text: str = ...,
        image: tkinter._Image | str = ...,
        values: list[Any] | tuple[Any, ...] = ...,
        open: bool = ...,
        tags: str | list[str] | tuple[str, ...] = ...,
    ) -> str:
        """Creates a new item and return the item identifier of the newly
        created item.

        parent is the item ID of the parent item, or the empty string
        to create a new top-level item. index is an integer, or the value
        end, specifying where in the list of parent's children to insert
        the new item. If index is less than or equal to zero, the new node
        is inserted at the beginning, if index is greater than or equal to
        the current number of children, it is inserted at the end. If iid
        is specified, it is used as the item identifier, iid must not
        already exist in the tree. Otherwise, a new unique identifier
        is generated.
        """

    @overload
    def item(self, item: str | int, option: Literal["text"]) -> str:
        """Query or modify the options for the specified item.

        If no options are given, a dict with options/values for the item
        is returned. If option is specified then the value for that option
        is returned. Otherwise, sets the options to the corresponding
        values as given by kw.
        """
    @overload
    def item(self, item: str | int, option: Literal["image"]) -> tuple[str] | Literal[""]: ...
    @overload
    def item(self, item: str | int, option: Literal["values"]) -> tuple[Any, ...] | Literal[""]: ...
    @overload
    def item(self, item: str | int, option: Literal["open"]) -> bool: ...  # actually 0 or 1
    @overload
    def item(self, item: str | int, option: Literal["tags"]) -> tuple[str, ...] | Literal[""]: ...
    @overload
    def item(self, item: str | int, option: str) -> Any: ...
    @overload
    def item(self, item: str | int, option: None = None) -> _TreeviewItemDict: ...
    @overload
    def item(
        self,
        item: str | int,
        option: None = None,
        *,
        text: str = ...,
        image: tkinter._Image | str = ...,
        values: list[Any] | tuple[Any, ...] | Literal[""] = ...,
        open: bool = ...,
        tags: str | list[str] | tuple[str, ...] = ...,
    ) -> None: ...

    def move(self, item: str | int, parent: str, index: int | Literal["end"]) -> None:
        """Moves item to position index in parent's list of children.

        It is illegal to move an item under one of its descendants. If
        index is less than or equal to zero, item is moved to the
        beginning, if greater than or equal to the number of children,
        it is moved to the end. If item was detached it is reattached.
        """

    reattach = move
    def next(self, item: str | int) -> str:  # returning empty string means last item
        """Returns the identifier of item's next sibling, or '' if item
        is the last child of its parent.
        """

    def parent(self, item: str | int) -> str:
        """Returns the ID of the parent of item, or '' if item is at the
        top level of the hierarchy.
        """

    def prev(self, item: str | int) -> str:  # returning empty string means first item
        """Returns the identifier of item's previous sibling, or '' if
        item is the first child of its parent.
        """

    def see(self, item: str | int) -> None:
        """Ensure that item is visible.

        Sets all of item's ancestors open option to True, and scrolls
        the widget if necessary so that item is within the visible
        portion of the tree.
        """

    def selection(self) -> tuple[str, ...]:
        """Returns the tuple of selected items."""

    @overload
    def selection_set(self, items: list[str] | tuple[str, ...] | list[int] | tuple[int, ...], /) -> None:
        """The specified items becomes the new selection."""
    @overload
    def selection_set(self, *items: str | int) -> None: ...

    @overload
    def selection_add(self, items: list[str] | tuple[str, ...] | list[int] | tuple[int, ...], /) -> None:
        """Add all of the specified items to the selection."""
    @overload
    def selection_add(self, *items: str | int) -> None: ...

    @overload
    def selection_remove(self, items: list[str] | tuple[str, ...] | list[int] | tuple[int, ...], /) -> None:
        """Remove all of the specified items from the selection."""
    @overload
    def selection_remove(self, *items: str | int) -> None: ...

    @overload
    def selection_toggle(self, items: list[str] | tuple[str, ...] | list[int] | tuple[int, ...], /) -> None:
        """Toggle the selection state of each specified item."""
    @overload
    def selection_toggle(self, *items: str | int) -> None: ...

    @overload
    def set(self, item: str | int, column: None = None, value: None = None) -> dict[str, Any]:
        """Query or set the value of given item.

        With one argument, return a dictionary of column/value pairs
        for the specified item. With two arguments, return the current
        value of the specified column. With three arguments, set the
        value of given column in given item to the specified value.
        """
    @overload
    def set(self, item: str | int, column: str | int, value: None = None) -> Any: ...
    @overload
    def set(self, item: str | int, column: str | int, value: Any) -> Literal[""]: ...

    # There's no tag_unbind() or 'add' argument for whatever reason.
    # Also, it's 'callback' instead of 'func' here.
    @overload
    def tag_bind(
        self, tagname: str, sequence: str | None = None, callback: Callable[[tkinter.Event[Treeview]], object] | None = None
    ) -> str:
        """Bind a callback for the given event sequence to the tag tagname.
        When an event is delivered to an item, the callbacks for each
        of the item's tags option are called.
        """
    @overload
    def tag_bind(self, tagname: str, sequence: str | None, callback: str) -> None: ...
    @overload
    def tag_bind(self, tagname: str, *, callback: str) -> None: ...

    @overload
    def tag_configure(self, tagname: str, option: Literal["foreground", "background"]) -> str:
        """Query or modify the options for the specified tagname.

        If kw is not given, returns a dict of the option settings for tagname.
        If option is specified, returns the value for that option for the
        specified tagname. Otherwise, sets the options to the corresponding
        values for the given tagname.
        """
    @overload
    def tag_configure(self, tagname: str, option: Literal["font"]) -> _FontDescription: ...
    @overload
    def tag_configure(self, tagname: str, option: Literal["image"]) -> str: ...
    @overload
    def tag_configure(
        self,
        tagname: str,
        option: None = None,
        *,
        # There is also 'text' and 'anchor', but they don't seem to do anything, using them is likely a bug
        foreground: str = ...,
        background: str = ...,
        font: _FontDescription = ...,
        image: tkinter._Image | str = ...,
    ) -> _TreeviewTagDict | MaybeNone: ...  # can be None but annoying to check

    @overload
    def tag_has(self, tagname: str, item: None = None) -> tuple[str, ...]:
        """If item is specified, returns True if the specified item has the
        given tagname, False otherwise. Otherwise, returns a list of
        all items which have the specified tag.

        * Availability: Tk 8.6
        """
    @overload
    def tag_has(self, tagname: str, item: str | int) -> bool: ...

class LabeledScale(Frame):
    """A Ttk Scale widget with a Ttk Label widget indicating its
    current value.

    The Ttk Scale can be accessed through instance.scale, and Ttk Label
    can be accessed through instance.label
    """

    label: Label
    scale: Scale
    # This should be kept in sync with tkinter.ttk.Frame.__init__()
    # (all the keyword-only args except compound are from there)
    def __init__(
        self,
        master: tkinter.Misc | None = None,
        variable: tkinter.IntVar | tkinter.DoubleVar | None = None,
        from_: float = 0,
        to: float = 10,
        *,
        border: float | str = ...,
        borderwidth: float | str = ...,
        class_: str = "",
        compound: Literal["top", "bottom"] = "top",
        cursor: tkinter._Cursor = "",
        height: float | str = 0,
        name: str = ...,
        padding: _Padding = ...,
        relief: Literal["raised", "sunken", "flat", "ridge", "solid", "groove"] = ...,
        style: str = "",
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = "",
        width: float | str = 0,
    ) -> None:
        """Construct a horizontal LabeledScale with parent master, a
        variable to be associated with the Ttk Scale widget and its range.
        If variable is not specified, a tkinter.IntVar is created.

        WIDGET-SPECIFIC OPTIONS

            compound: 'top' or 'bottom'
                Specifies how to display the label relative to the scale.
                Defaults to 'top'.
        """

    # destroy is overridden, signature does not change
    value: Any
    """Return current scale value."""

class OptionMenu(Menubutton):
    """Themed OptionMenu, based after tkinter's OptionMenu, which allows
    the user to select a value from a menu.
    """

    if sys.version_info >= (3, 14):
        def __init__(
            self,
            master: tkinter.Misc | None,
            variable: tkinter.StringVar,
            default: str | None = None,
            *values: str,
            # rest of these are keyword-only because *args syntax used above
            style: str = "",
            direction: Literal["above", "below", "left", "right", "flush"] = "below",
            command: Callable[[tkinter.StringVar], object] | None = None,
            name: str | None = None,
        ) -> None:
            """Construct a themed OptionMenu widget with master as the parent,
            the option textvariable set to variable, the initially selected
            value specified by the default parameter, the menu values given by
            *values and additional keywords.

            WIDGET-SPECIFIC OPTIONS

                style: stylename
                    Menubutton style.
                direction: 'above', 'below', 'left', 'right', or 'flush'
                    Menubutton direction.
                command: callback
                    A callback that will be invoked after selecting an item.
            """

    else:
        def __init__(
            self,
            master: tkinter.Misc | None,
            variable: tkinter.StringVar,
            default: str | None = None,
            *values: str,
            # rest of these are keyword-only because *args syntax used above
            style: str = "",
            direction: Literal["above", "below", "left", "right", "flush"] = "below",
            command: Callable[[tkinter.StringVar], object] | None = None,
        ) -> None:
            """Construct a themed OptionMenu widget with master as the parent,
            the option textvariable set to variable, the initially selected
            value specified by the default parameter, the menu values given by
            *values and additional keywords.

            WIDGET-SPECIFIC OPTIONS

                style: stylename
                    Menubutton style.
                direction: 'above', 'below', 'left', 'right', or 'flush'
                    Menubutton direction.
                command: callback
                    A callback that will be invoked after selecting an item.
            """

    # configure, config, cget, destroy are inherited from Menubutton
    # destroy and __setitem__ are overridden, signature does not change
    def set_menu(self, default: str | None = None, *values: str) -> None:
        """Build a new menu of radiobuttons with *values and optionally
        a default value.
        """



import sys
from _typeshed import ReadableBuffer
from typing import final
from typing_extensions import Self

@final
class BZ2Compressor:
    """Create a compressor object for compressing data incrementally.

      compresslevel
        Compression level, as a number between 1 and 9.

    For one-shot compression, use the compress() function instead.
    """

    if sys.version_info >= (3, 12):
        def __new__(cls, compresslevel: int = 9, /) -> Self: ...
    else:
        def __init__(self, compresslevel: int = 9, /) -> None: ...

    def compress(self, data: ReadableBuffer, /) -> bytes:
        """Provide data to the compressor object.

        Returns a chunk of compressed data if possible, or b'' otherwise.

        When you have finished providing data to the compressor, call the
        flush() method to finish the compression process.
        """

    def flush(self) -> bytes:
        """Finish the compression process.

        Returns the compressed data left in internal buffers.

        The compressor object may not be used after this method is called.
        """

@final
class BZ2Decompressor:
    """Create a decompressor object for decompressing data incrementally.

    For one-shot decompression, use the decompress() function instead.
    """

    def decompress(self, data: ReadableBuffer, max_length: int = -1) -> bytes:
        """Decompress *data*, returning uncompressed data as bytes.

        If *max_length* is nonnegative, returns at most *max_length* bytes
        of decompressed data.  If this limit is reached and further output
        can be produced, *self.needs_input* will be set to ``False``.  In
        this case, the next call to *decompress()* may provide *data* as b''
        to obtain more of the output.

        If all of the input data was decompressed and returned (either
        because this was less than *max_length* bytes, or because
        *max_length* was negative), *self.needs_input* will be set to True.

        Attempting to decompress data after the end of stream is reached
        raises an EOFError.  Any data found after the end of the stream is
        ignored and saved in the unused_data attribute.
        """

    @property
    def eof(self) -> bool:
        """True if the end-of-stream marker has been reached."""

    @property
    def needs_input(self) -> bool:
        """True if more input is needed before more decompressed data can be produced."""

    @property
    def unused_data(self) -> bytes:
        """Data found after the end of the compressed stream."""



"""Fixer for function definitions with tuple parameters.

def func(((a, b), c), d):
    ...

    ->

def func(x, d):
    ((a, b), c) = x
    ...

It will also support lambdas:

    lambda (x, y): x + y -> lambda t: t[0] + t[1]

    # The parens are a syntax error in Python 3
    lambda (x): x + y -> lambda x: x + y
"""

from typing import ClassVar, Literal

from .. import fixer_base

def is_docstring(stmt): ...

class FixTupleParams(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[True]]
    PATTERN: ClassVar[str]
    def transform(self, node, results): ...
    def transform_lambda(self, node, results) -> None: ...

def simplify_args(node): ...
def find_params(node): ...
def map_to_index(param_list, prefix=[], d=None): ...
def tuple_name(param_list): ...



"""Append module search paths for third-party packages to sys.path.

****************************************************************
* This module is automatically imported during initialization. *
****************************************************************

This will append site-specific paths to the module search path.  On
Unix (including Mac OSX), it starts with sys.prefix and
sys.exec_prefix (if different) and appends
lib/python<version>/site-packages.
On other platforms (such as Windows), it tries each of the
prefixes directly, as well as with lib/site-packages appended.  The
resulting directories, if they exist, are appended to sys.path, and
also inspected for path configuration files.

If a file named "pyvenv.cfg" exists one directory above sys.executable,
sys.prefix and sys.exec_prefix are set to that directory and
it is also checked for site-packages (sys.base_prefix and
sys.base_exec_prefix will always be the "real" prefixes of the Python
installation). If "pyvenv.cfg" (a bootstrap configuration file) contains
the key "include-system-site-packages" set to "true" (case-insensitive),
the system-level prefixes will still also be searched for site-packages;
otherwise they won't.

Two kinds of configuration files are processed in each site-packages
directory:

- <name>.pth files extend sys.path with additional directories (one per
  line).  Lines starting with "import" are deprecated (see PEP 829).

- <name>.start files specify startup entry points using the pkg.mod:callable
  syntax.  These are resolved via pkgutil.resolve_name() and called with no
  arguments.

When called from main(), all .pth path extensions are applied before any
.start entry points are executed, ensuring that paths are available before
startup code runs.

See the documentation for the site module for full details:
https://docs.python.org/3/library/site.html
"""

import sys
from _typeshed import StrPath
from collections.abc import Iterable

PREFIXES: list[str]
ENABLE_USER_SITE: bool | None
USER_SITE: str | None
USER_BASE: str | None

def main() -> None:
    """Add standard site-specific directories to the module search path.

    This function is called automatically when this module is imported,
    unless the python interpreter was started with the -S flag.
    """

def abs_paths() -> None:  # undocumented
    """Set __file__ to an absolute path."""

def addpackage(sitedir: StrPath, name: StrPath, known_paths: set[str] | None) -> set[str] | None:  # undocumented
    """Process a .pth file within the site-packages directory."""

if sys.version_info >= (3, 15):
    class StartupState:
        """Per-batch accumulator for .pth and .start file processing.

        A StartupState collects sys.path extensions, deprecated .pth import lines,
        and .start entry points read from one or more site-packages directories.
        Calling process() applies them in PEP 829 order: paths are added to
        sys.path first, then import lines from .pth files (skipping any with a
        matching .start), then entry points from .start files.

        State lives entirely on the instance; there is no module-level pending
        state.  This is what makes the module reentrancy-safe: a site.addsitedir()
        call reached recursively from an exec'd import line or a .start entry
        point operates on a different StartupState than the one being processed by
        the outer call.

        The internal data is intentionally private.  The lower-level write
        methods (_record_sitedir(), _read_pth_file(), _read_start_file()) are
        private to the site module; the public surface is addsitedir(),
        addusersitepackages(), addsitepackages(), and process().
        """

        __slots__ = ("_known_paths", "_processed_sitedirs", "_path_entries", "_importexecs", "_entrypoints")
        def __init__(self, known_paths: set[str] | None = None) -> None:
            """Create an independent startup state.

            *known_paths* is a set of case-normalized paths already present
            on sys.path, used to avoid duplicate path entries.  When None
            (the default), it is initialized from the current sys.path.

            A caller-supplied set is stored by reference and mutated in place
            as new paths are recorded; pass a fresh set per StartupState if
            isolation across instances is required.
            """

        def addsitedir(self, sitedir: str) -> None:
            """Add a site directory and accumulate its .pth and .start startup data.

            Read the .pth and .start files in *sitedir* and record their
            sys.path extensions, deprecated .pth import lines, and .start entry
            points on this state.  The recorded data is not applied until
            process() is called.

            Typically used to batch multiple site directories before a single
            process() call, so that every sys.path extension is visible before
            any startup code runs.  Reentrant calls reached from a .start entry
            point or an exec'd .pth import line must not mutate the state
            currently being processed; for those cases, use site.addsitedir()
            instead, which always creates a fresh per-call state.
            """

        def addusersitepackages(self) -> None:
            """Add the per-user site-packages directory, if enabled.

            The user site directory is added only when user site-packages are
            enabled and the directory exists.  Its startup data is accumulated
            for later processing by process().
            """

        def addsitepackages(self, prefixes: Iterable[str] | None = None) -> None:
            """Add global site-packages directories, if they exist.

            Site-packages directories are computed from *prefixes*, or from the
            global PREFIXES when *prefixes* is None.  Each directory's startup
            data is accumulated for later processing by process().
            """

        def process(self) -> None:
            """Apply accumulated state in PEP 829 order.

            Phase order matters: all .pth path extensions are applied to
            sys.path *before* any import line or .start entry point runs, so
            that an entry point may live in a module reachable only via a
            .pth-extended path.
            """

def addsitedir(sitedir: str, known_paths: set[str] | None = None) -> None:
    """Add a site directory and process its startup files.

    For batched processing across multiple site directories, build a
    StartupState explicitly and call StartupState.addsitedir() on it; that
    defers .pth/.start processing until a single StartupState.process() call.
    """

def addsitepackages(known_paths: set[str] | None, prefixes: Iterable[str] | None = None) -> set[str] | None:  # undocumented
    """Add global site-packages directories, if they exist.

    Site-packages directories are computed from *prefixes*, or from the global
    prefixes when *prefixes* is None.  Return *known_paths*, updated with any
    paths added by addsitedir().
    """

def addusersitepackages(known_paths: set[str] | None) -> set[str] | None:  # undocumented
    """Add the per-user site-packages directory, if enabled.

    The user site directory is added only when user site-packages are enabled
    and the directory exists.  Return *known_paths*, updated with any paths
    added by addsitedir().
    """

def check_enableusersite() -> bool | None:  # undocumented
    """Check if user site directory is safe for inclusion

    The function tests for the command line flag (including environment var),
    process uid/gid equal to effective uid/gid.

    None: Disabled for security reasons
    False: Disabled by user (command line option)
    True: Safe and enabled
    """

if sys.version_info >= (3, 13):
    def gethistoryfile() -> str:  # undocumented
        """Check if the PYTHON_HISTORY environment variable is set and define
        it as the .python_history file.  If PYTHON_HISTORY is not set, use the
        default .python_history file.
        """

def enablerlcompleter() -> None:  # undocumented
    """Enable default readline configuration on interactive prompts, by
    registering a sys.__interactivehook__.
    """

if sys.version_info >= (3, 13):
    def register_readline() -> None:  # undocumented
        """Configure readline completion on interactive prompts.

        If the readline module can be imported, the hook will set the Tab key
        as completion key and register ~/.python_history as history file.
        This can be overridden in the sitecustomize or usercustomize module,
        or in a PYTHONSTARTUP file.
        """

def execsitecustomize() -> None:  # undocumented
    """Run custom site specific code, if available."""

def execusercustomize() -> None:  # undocumented
    """Run custom user specific code, if available."""

def getsitepackages(prefixes: Iterable[str] | None = None) -> list[str]:
    """Returns a list containing all global site-packages directories.

    For each directory present in ``prefixes`` (or the global ``PREFIXES``),
    this function will find its `site-packages` subdirectory depending on the
    system environment, and will return a list of full paths.
    """

def getuserbase() -> str:
    """Returns the `user base` directory path.

    The `user base` directory can be used to store data. If the global
    variable ``USER_BASE`` is not initialized yet, this function will also set
    it.
    """

def getusersitepackages() -> str:
    """Returns the user-specific site-packages directory path.

    If the global variable ``USER_SITE`` is not initialized yet, this
    function will also set it.
    """

def makepath(*paths: StrPath) -> tuple[str, str]: ...  # undocumented
def removeduppaths() -> set[str]:  # undocumented
    """Remove duplicate entries from sys.path along with making them
    absolute
    """

def setcopyright() -> None:  # undocumented
    """Set 'copyright' and 'credits' in builtins"""

def sethelper() -> None: ...  # undocumented
def setquit() -> None:  # undocumented
    """Define new builtins 'quit' and 'exit'.

    These are objects which make the interpreter exit when called.
    The repr of each object contains a hint at how it works.

    """

def venv(known_paths: set[str] | None) -> set[str] | None:  # undocumented
    """Process pyvenv.cfg and add the venv site-packages, if applicable."""



"""Implementation of JSONEncoder"""

from collections.abc import Callable, Iterator
from re import Pattern
from typing import Any, Final

ESCAPE: Final[Pattern[str]]  # undocumented
ESCAPE_ASCII: Final[Pattern[str]]  # undocumented
HAS_UTF8: Final[Pattern[bytes]]  # undocumented
ESCAPE_DCT: Final[dict[str, str]]  # undocumented
INFINITY: Final[float]  # undocumented

def py_encode_basestring(s: str) -> str:  # undocumented
    """Return a JSON representation of a Python string"""

def py_encode_basestring_ascii(s: str) -> str:  # undocumented
    """Return an ASCII-only JSON representation of a Python string"""

def encode_basestring(s: str, /) -> str:  # undocumented
    """Return a JSON representation of a Python string"""

def encode_basestring_ascii(s: str, /) -> str:  # undocumented
    """Return an ASCII-only JSON representation of a Python string"""

class JSONEncoder:
    """Extensible JSON <https://json.org> encoder for Python data structures.

    Supports the following objects and types by default:

    +-------------------+---------------+
    | Python            | JSON          |
    +===================+===============+
    | dict, frozendict  | object        |
    +-------------------+---------------+
    | list, tuple       | array         |
    +-------------------+---------------+
    | str               | string        |
    +-------------------+---------------+
    | int, float        | number        |
    +-------------------+---------------+
    | True              | true          |
    +-------------------+---------------+
    | False             | false         |
    +-------------------+---------------+
    | None              | null          |
    +-------------------+---------------+

    To extend this to recognize other objects, subclass and implement a
    ``.default()`` method with another method that returns a serializable
    object for ``o`` if possible, otherwise it should call the superclass
    implementation (to raise ``TypeError``).

    """

    item_separator: str
    key_separator: str

    skipkeys: bool
    ensure_ascii: bool
    check_circular: bool
    allow_nan: bool
    sort_keys: bool
    indent: int | str
    def __init__(
        self,
        *,
        skipkeys: bool = False,
        ensure_ascii: bool = True,
        check_circular: bool = True,
        allow_nan: bool = True,
        sort_keys: bool = False,
        indent: int | str | None = None,
        separators: tuple[str, str] | None = None,
        default: Callable[..., Any] | None = None,
    ) -> None:
        """Constructor for JSONEncoder, with sensible defaults.

        If skipkeys is false, then it is a TypeError to attempt
        encoding of keys that are not str, int, float, bool or None.
        If skipkeys is True, such items are simply skipped.

        If ensure_ascii is true, the output is guaranteed to be str objects
        with all incoming non-ASCII and non-printable characters escaped.
        If ensure_ascii is false, the output can contain non-ASCII and
        non-printable characters.

        If check_circular is true, then lists, dicts, and custom encoded
        objects will be checked for circular references during encoding to
        prevent an infinite recursion (which would cause an RecursionError).
        Otherwise, no such check takes place.

        If allow_nan is true, then NaN, Infinity, and -Infinity will be
        encoded as such.  This behavior is not JSON specification compliant,
        but is consistent with most JavaScript based encoders and decoders.
        Otherwise, it will be a ValueError to encode such floats.

        If sort_keys is true, then the output of dictionaries will be
        sorted by key; this is useful for regression tests to ensure
        that JSON serializations can be compared on a day-to-day basis.

        If indent is a non-negative integer, then JSON array
        elements and object members will be pretty-printed with that
        indent level.  An indent level of 0 will only insert newlines.
        None is the most compact representation.

        If specified, separators should be an (item_separator,
        key_separator) tuple.  The default is (', ', ': ') if *indent* is
        ``None`` and (',', ': ') otherwise.  To get the most compact JSON
        representation, you should specify (',', ':') to eliminate
        whitespace.

        If specified, default is a function that gets called for objects
        that can't otherwise be serialized.  It should return a JSON
        encodable version of the object or raise a ``TypeError``.

        """

    def default(self, o: Any) -> Any:
        """Implement this method in a subclass such that it returns
        a serializable object for ``o``, or calls the base implementation
        (to raise a ``TypeError``).

        For example, to support arbitrary iterators, you could
        implement default like this::

            def default(self, o):
                try:
                    iterable = iter(o)
                except TypeError:
                    pass
                else:
                    return list(iterable)
                # Let the base class default method raise the TypeError
                return super().default(o)

        """

    def encode(self, o: Any) -> str:
        """Return a JSON string representation of a Python data structure.

        >>> from json.encoder import JSONEncoder
        >>> JSONEncoder().encode({"foo": ["bar", "baz"]})
        '{"foo": ["bar", "baz"]}'

        """

    def iterencode(self, o: Any, _one_shot: bool = False) -> Iterator[str]:
        """Encode the given object and yield each string
        representation as available.

        For example::

            for chunk in JSONEncoder().iterencode(bigobject):
                mysocket.write(chunk)

        """



import ssl
import sys
from collections import deque
from collections.abc import Callable
from enum import Enum
from typing import Any, ClassVar, Final, Literal, TypeAlias

from . import constants, events, futures, protocols, transports

def _create_transport_context(server_side: bool, server_hostname: str | None) -> ssl.SSLContext: ...

if sys.version_info >= (3, 11):
    SSLAgainErrors: tuple[type[ssl.SSLWantReadError], type[ssl.SSLSyscallError]]

    class SSLProtocolState(Enum):
        UNWRAPPED = "UNWRAPPED"
        DO_HANDSHAKE = "DO_HANDSHAKE"
        WRAPPED = "WRAPPED"
        FLUSHING = "FLUSHING"
        SHUTDOWN = "SHUTDOWN"

    class AppProtocolState(Enum):
        STATE_INIT = "STATE_INIT"
        STATE_CON_MADE = "STATE_CON_MADE"
        STATE_EOF = "STATE_EOF"
        STATE_CON_LOST = "STATE_CON_LOST"

    def add_flowcontrol_defaults(high: int | None, low: int | None, kb: int) -> tuple[int, int]: ...

else:
    _UNWRAPPED: Final = "UNWRAPPED"
    _DO_HANDSHAKE: Final = "DO_HANDSHAKE"
    _WRAPPED: Final = "WRAPPED"
    _SHUTDOWN: Final = "SHUTDOWN"

if sys.version_info < (3, 11):
    class _SSLPipe:
        """An SSL "Pipe".

        An SSL pipe allows you to communicate with an SSL/TLS protocol instance
        through memory buffers. It can be used to implement a security layer for an
        existing connection where you don't have access to the connection's file
        descriptor, or for some reason you don't want to use it.

        An SSL pipe can be in "wrapped" and "unwrapped" mode. In unwrapped mode,
        data is passed through untransformed. In wrapped mode, application level
        data is encrypted to SSL record level data and vice versa. The SSL record
        level is the lowest level in the SSL protocol suite and is what travels
        as-is over the wire.

        An SslPipe initially is in "unwrapped" mode. To start SSL, call
        do_handshake(). To shutdown SSL again, call unwrap().
        """

        max_size: ClassVar[int]

        _context: ssl.SSLContext
        _server_side: bool
        _server_hostname: str | None
        _state: str
        _incoming: ssl.MemoryBIO
        _outgoing: ssl.MemoryBIO
        _sslobj: ssl.SSLObject | None
        _need_ssldata: bool
        _handshake_cb: Callable[[BaseException | None], None] | None
        _shutdown_cb: Callable[[], None] | None
        def __init__(self, context: ssl.SSLContext, server_side: bool, server_hostname: str | None = None) -> None:
            """
            The *context* argument specifies the ssl.SSLContext to use.

            The *server_side* argument indicates whether this is a server side or
            client side transport.

            The optional *server_hostname* argument can be used to specify the
            hostname you are connecting to. You may only specify this parameter if
            the _ssl module supports Server Name Indication (SNI).
            """

        @property
        def context(self) -> ssl.SSLContext:
            """The SSL context passed to the constructor."""

        @property
        def ssl_object(self) -> ssl.SSLObject | None:
            """The internal ssl.SSLObject instance.

            Return None if the pipe is not wrapped.
            """

        @property
        def need_ssldata(self) -> bool:
            """Whether more record level data is needed to complete a handshake
            that is currently in progress.
            """

        @property
        def wrapped(self) -> bool:
            """
            Whether a security layer is currently in effect.

            Return False during handshake.
            """

        def do_handshake(self, callback: Callable[[BaseException | None], object] | None = None) -> list[bytes]:
            """Start the SSL handshake.

            Return a list of ssldata. A ssldata element is a list of buffers

            The optional *callback* argument can be used to install a callback that
            will be called when the handshake is complete. The callback will be
            called with None if successful, else an exception instance.
            """

        def shutdown(self, callback: Callable[[], object] | None = None) -> list[bytes]:
            """Start the SSL shutdown sequence.

            Return a list of ssldata. A ssldata element is a list of buffers

            The optional *callback* argument can be used to install a callback that
            will be called when the shutdown is complete. The callback will be
            called without arguments.
            """

        def feed_eof(self) -> None:
            """Send a potentially "ragged" EOF.

            This method will raise an SSL_ERROR_EOF exception if the EOF is
            unexpected.
            """

        def feed_ssldata(self, data: bytes, only_handshake: bool = False) -> tuple[list[bytes], list[bytes]]:
            """Feed SSL record level data into the pipe.

            The data must be a bytes instance. It is OK to send an empty bytes
            instance. This can be used to get ssldata for a handshake initiated by
            this endpoint.

            Return a (ssldata, appdata) tuple. The ssldata element is a list of
            buffers containing SSL data that needs to be sent to the remote SSL.

            The appdata element is a list of buffers containing plaintext data that
            needs to be forwarded to the application. The appdata list may contain
            an empty buffer indicating an SSL "close_notify" alert. This alert must
            be acknowledged by calling shutdown().
            """

        def feed_appdata(self, data: bytes, offset: int = 0) -> tuple[list[bytes], int]:
            """Feed plaintext data into the pipe.

            Return an (ssldata, offset) tuple. The ssldata element is a list of
            buffers containing record level data that needs to be sent to the
            remote SSL instance. The offset is the number of plaintext bytes that
            were processed, which may be less than the length of data.

            NOTE: In case of short writes, this call MUST be retried with the SAME
            buffer passed into the *data* argument (i.e. the id() must be the
            same). This is an OpenSSL requirement. A further particularity is that
            a short write will always have offset == 0, because the _ssl module
            does not enable partial writes. And even though the offset is zero,
            there will still be encrypted data in ssldata.
            """

class _SSLProtocolTransport(transports._FlowControlMixin, transports.Transport):
    _sendfile_compatible: ClassVar[constants._SendfileMode]

    _loop: events.AbstractEventLoop
    if sys.version_info >= (3, 11):
        _ssl_protocol: SSLProtocol | None
    else:
        _ssl_protocol: SSLProtocol
    _closed: bool
    def __init__(self, loop: events.AbstractEventLoop, ssl_protocol: SSLProtocol) -> None: ...
    def get_extra_info(self, name: str, default: Any | None = None) -> dict[str, Any]:
        """Get optional transport information."""

    @property
    def _protocol_paused(self) -> bool: ...
    def write(self, data: bytes | bytearray | memoryview[Any]) -> None:  # any memoryview format or shape
        """Write some data bytes to the transport.

        This does not block; it buffers the data and arranges for it
        to be sent out asynchronously.
        """

    def can_write_eof(self) -> Literal[False]:
        """Return True if this transport supports write_eof(), False if not."""

    if sys.version_info >= (3, 11):
        def get_write_buffer_limits(self) -> tuple[int, int]: ...
        def get_read_buffer_limits(self) -> tuple[int, int]: ...
        def set_read_buffer_limits(self, high: int | None = None, low: int | None = None) -> None:
            """Set the high- and low-water limits for read flow control.

            These two values control when to call the upstream transport's
            pause_reading() and resume_reading() methods.  If specified,
            the low-water limit must be less than or equal to the
            high-water limit.  Neither value can be negative.

            The defaults are implementation-specific.  If only the
            high-water limit is given, the low-water limit defaults to an
            implementation-specific value less than or equal to the
            high-water limit.  Setting high to zero forces low to zero as
            well, and causes pause_reading() to be called whenever the
            buffer becomes non-empty.  Setting low to zero causes
            resume_reading() to be called only once the buffer is empty.
            Use of zero for either limit is generally sub-optimal as it
            reduces opportunities for doing I/O and computation
            concurrently.
            """

        def get_read_buffer_size(self) -> int:
            """Return the current size of the read buffer."""

    def __del__(self) -> None: ...

if sys.version_info >= (3, 11):
    _SSLProtocolBase: TypeAlias = protocols.BufferedProtocol
else:
    _SSLProtocolBase: TypeAlias = protocols.Protocol

class SSLProtocol(_SSLProtocolBase):
    """SSL protocol.

    Implementation of SSL on top of a socket using incoming and outgoing
    buffers which are ssl.MemoryBIO objects.
    """

    _server_side: bool
    _server_hostname: str | None
    _sslcontext: ssl.SSLContext
    _extra: dict[str, Any]
    _write_backlog: deque[tuple[bytes, int]]
    _write_buffer_size: int
    _waiter: futures.Future[Any]
    _loop: events.AbstractEventLoop
    _app_transport: _SSLProtocolTransport
    _transport: transports.BaseTransport | None
    _ssl_handshake_timeout: int | None
    _app_protocol: protocols.BaseProtocol
    _app_protocol_is_buffer: bool

    if sys.version_info >= (3, 11):
        max_size: ClassVar[int]
    else:
        _sslpipe: _SSLPipe | None
        _session_established: bool
        _call_connection_made: bool
        _in_handshake: bool
        _in_shutdown: bool

    if sys.version_info >= (3, 11):
        def __init__(
            self,
            loop: events.AbstractEventLoop,
            app_protocol: protocols.BaseProtocol,
            sslcontext: ssl.SSLContext,
            waiter: futures.Future[Any],
            server_side: bool = False,
            server_hostname: str | None = None,
            call_connection_made: bool = True,
            ssl_handshake_timeout: int | None = None,
            ssl_shutdown_timeout: float | None = None,
        ) -> None: ...
    else:
        def __init__(
            self,
            loop: events.AbstractEventLoop,
            app_protocol: protocols.BaseProtocol,
            sslcontext: ssl.SSLContext,
            waiter: futures.Future[Any],
            server_side: bool = False,
            server_hostname: str | None = None,
            call_connection_made: bool = True,
            ssl_handshake_timeout: int | None = None,
        ) -> None: ...

    def _set_app_protocol(self, app_protocol: protocols.BaseProtocol) -> None: ...
    def _wakeup_waiter(self, exc: BaseException | None = None) -> None: ...
    def connection_lost(self, exc: BaseException | None) -> None:
        """Called when the low-level connection is lost or closed.

        The argument is an exception object or None (the latter
        meaning a regular EOF is received or the connection was
        aborted or closed).
        """

    def eof_received(self) -> None:
        """Called when the other end of the low-level stream
        is half-closed.

        If this returns a false value (including None), the transport
        will close itself.  If it returns a true value, closing the
        transport is up to the protocol.
        """

    def _get_extra_info(self, name: str, default: Any | None = None) -> Any: ...
    def _start_shutdown(self) -> None: ...
    if sys.version_info >= (3, 11):
        def _write_appdata(self, list_of_data: list[bytes]) -> None: ...
    else:
        def _write_appdata(self, data: bytes) -> None: ...

    def _start_handshake(self) -> None: ...
    def _check_handshake_timeout(self) -> None: ...
    def _on_handshake_complete(self, handshake_exc: BaseException | None) -> None: ...
    def _fatal_error(self, exc: BaseException, message: str = "Fatal error on transport") -> None: ...
    if sys.version_info >= (3, 11):
        def _abort(self, exc: BaseException | None) -> None: ...
        def get_buffer(self, n: int) -> memoryview: ...
    else:
        def _abort(self) -> None: ...
        def _finalize(self) -> None: ...
        def _process_write_backlog(self) -> None: ...



"""Heap queue algorithm (a.k.a. priority queue).

Heaps are arrays for which a[k] <= a[2*k+1] and a[k] <= a[2*k+2] for
all k, counting elements from 0.  For the sake of comparison,
non-existing elements are considered to be infinite.  The interesting
property of a heap is that a[0] is always its smallest element.

Usage:

heap = []            # creates an empty heap
heappush(heap, item) # pushes a new item on the heap
item = heappop(heap) # pops the smallest item from the heap
item = heap[0]       # smallest item on the heap without popping it
heapify(x)           # transforms list into a heap, in-place, in linear time
item = heapreplace(heap, item) # pops and returns smallest item, and adds
                               # new item; the heap size is unchanged

Our API differs from textbook heap algorithms as follows:

- We use 0-based indexing.  This makes the relationship between the
  index for a node and the indexes for its children slightly less
  obvious, but is more suitable since Python uses 0-based indexing.

- Our heappop() method returns the smallest item, not the largest.

These two make it possible to view the heap as a regular Python list
without surprises: heap[0] is the smallest item, and heap.sort()
maintains the heap invariant!
"""

import sys
from _typeshed import SupportsRichComparisonT as _T  # All type variable use in this module requires comparability.
from typing import Final

__about__: Final[str]

def heapify(heap: list[_T], /) -> None:  # To work around the fact that list is invariant
    """Transform list into a heap, in-place, in O(len(heap)) time."""

def heappop(heap: list[_T], /) -> _T:
    """Pop the smallest item off the heap, maintaining the heap invariant."""

def heappush(heap: list[_T], item: _T, /) -> None:
    """Push item onto heap, maintaining the heap invariant."""

def heappushpop(heap: list[_T], item: _T, /) -> _T:
    """Push item on the heap, then pop and return the smallest item from the heap.

    The combined action runs more efficiently than heappush() followed by
    a separate call to heappop().
    """

def heapreplace(heap: list[_T], item: _T, /) -> _T:
    """Pop and return the current smallest value, and add the new item.

    This is more efficient than heappop() followed by heappush(), and can be
    more appropriate when using a fixed-size heap.  Note that the value
    returned may be larger than item!  That constrains reasonable uses of
    this routine unless written as part of a conditional replacement:

        if item > heap[0]:
            item = heapreplace(heap, item)
    """

if sys.version_info >= (3, 14):
    def heapify_max(heap: list[_T], /) -> None:
        """Maxheap variant of heapify."""

    def heappop_max(heap: list[_T], /) -> _T:
        """Maxheap variant of heappop."""

    def heappush_max(heap: list[_T], item: _T, /) -> None:
        """Push item onto max heap, maintaining the heap invariant."""

    def heappushpop_max(heap: list[_T], item: _T, /) -> _T:
        """Maxheap variant of heappushpop.

        The combined action runs more efficiently than heappush_max()
        followed by a separate call to heappop_max().
        """

    def heapreplace_max(heap: list[_T], item: _T, /) -> _T:
        """Maxheap variant of heapreplace."""



l = [1,2,3,4]


def foo():
    yield from l # some comment

    # weird indents
    yield\
                    from\
                        l
                    # indented trailing comment

    a = yield from l

    with (
        # Comment
        yield from l
        # Comment
    ):
        pass

    c = [(yield from l) , (
        yield from l

    )]

    while (
        yield from l
    ):
        pass

    yield (
        yield from l
    )




"""Fast profiler"""

from _typeshed import structseq
from collections.abc import Callable
from types import CodeType
from typing import Any, Final, final
from typing_extensions import disjoint_base

@disjoint_base
class Profiler:
    """Build a profiler object using the specified timer function.

    The default timer is a fast built-in one based on real time.
    For custom timer functions returning integers, 'timeunit' can
    be a float specifying a scale (that is, how long each integer unit
    is, in seconds).
    """

    def __init__(
        self, timer: Callable[[], float] | None = None, timeunit: float = 0.0, subcalls: bool = True, builtins: bool = True
    ) -> None: ...
    def getstats(self) -> list[profiler_entry]:
        """list of profiler_entry objects.

        getstats() -> list of profiler_entry objects

        Return all information collected by the profiler.
        Each profiler_entry is a tuple-like object with the
        following attributes:

            code          code object
            callcount     how many times this was called
            reccallcount  how many times called recursively
            totaltime     total time in this entry
            inlinetime    inline time in this entry (not in subcalls)
            calls         details of the calls

        The calls attribute is either None or a list of
        profiler_subentry objects:

            code          called code object
            callcount     how many times this is called
            reccallcount  how many times this is called recursively
            totaltime     total time spent in this call
            inlinetime    inline time (not in further subcalls)
        """

    def enable(self, subcalls: bool = True, builtins: bool = True) -> None:
        """Start collecting profiling information.

        subcalls
          If True, also records for each function
          statistics separated according to its current caller.
        builtins
          If True, records the time spent in
          built-in functions separately from their caller.
        """

    def disable(self) -> None:
        """Stop collecting profiling information."""

    def clear(self) -> None:
        """Clear all profiling information collected so far."""

@final
class profiler_entry(structseq[Any], tuple[CodeType | str, int, int, float, float, list[profiler_subentry]]):
    __match_args__: Final = ("code", "callcount", "reccallcount", "totaltime", "inlinetime", "calls")
    code: CodeType | str
    """code object or built-in function name"""

    callcount: int
    """how many times this was called"""

    reccallcount: int
    """how many times called recursively"""

    totaltime: float
    """total time in this entry"""

    inlinetime: float
    """inline time in this entry (not in subcalls)"""

    calls: list[profiler_subentry]
    """details of the calls"""

@final
class profiler_subentry(structseq[Any], tuple[CodeType | str, int, int, float, float]):
    __match_args__: Final = ("code", "callcount", "reccallcount", "totaltime", "inlinetime")
    code: CodeType | str
    """called code object or built-in function name"""

    callcount: int
    """how many times this is called"""

    reccallcount: int
    """how many times this is called recursively"""

    totaltime: float
    """total time spent in this call"""

    inlinetime: float
    """inline time (not in further subcalls)"""
